<?php

namespace Drupal\media_download;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;

use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;

use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Adds support for direct downloads of media entities.
 *
 * Copyright (C) 2021  Library Solutions, LLC (et al.).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
class DownloadController extends ControllerBase {

  /**
   * Attempt to retrieve a file from the supplied media.
   *
   * This method will only return a file if all of the following conditions are
   * satisfied by the supplied media entity:
   *
   * 1. A source field can be determined
   * 2. The source field exists on the supplied media entity
   * 3. At least one field delta references a file that exists on disk
   *
   * @return \Drupal\file\FileInterface|null
   *   A file on success, otherwise FALSE.
   *
   * @throws \RuntimeException
   *   If the supplied media entity is NULL, or has no source field.
   */
  protected function getFile(MediaInterface $media) {
    // Ensure that the source field can be determined for this media.
    if (!isset($media) || empty($source_field = $media->getSource()->getConfiguration()['source_field'] ?? NULL) || !$media->hasField($source_field)) {
      throw new \RuntimeException("No source field available for the requested entity.");
    }

    // Attempt to locate a valid file in the source field.
    foreach ($media->{$source_field} as $item) {
      // Skip this field item if it doesn't reference a file entity.
      if (!$item->entity instanceof FileInterface) {
        continue;
      }

      // Skip this field item if the referenced file doesn't exist.
      if (!file_exists($item->entity->getFileUri())) {
        continue;
      }

      return $item->entity;
    }

    return NULL;
  }

  /**
   * Download the primary file resource referenced by the supplied media entity.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity for which to initiate a file download.
   *
   * @return \Drupal\media_download\CacheableBinaryFileResponse
   *   A cacheable binary file response.
   */
  public function save(MediaInterface $media) {
    $cacheability = new CacheableMetadata();

    // Ensure that a file is referenced by the source field, and it exists.
    if (empty($file = $this->getFile($media))) {
      throw new NotFoundHttpException("There is no file associated with the requested entity.");
    }

    // Add both the media entity and the file entity that it references as
    // cacheable dependencies for this response.
    $cacheability->addCacheableDependency($file);
    $cacheability->addCacheableDependency($media);

    // Create a new response object to download the requested file.
    $response = new CacheableBinaryFileResponse($file->getFileUri());
    $response->addCacheableDependency($cacheability);
    $response->setContentDisposition('inline');

    // Clear the Cache-Control header so it can be calculated automatically.
    $response->headers->set('Cache-Control', '');

    // Automatically calculate the ETag and Last-Modified headers.
    $response->setAutoEtag();
    $response->setAutoLastModified();

    return $response;
  }

}
